package net;
import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.SocketException;
import java.nio.ByteBuffer;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
public class Transport implements TransportInterface{
	private final int BUFFER_SIZE = 20;
	private DatagramSocket socketEntrada;
	private DatagramSocket socketSaida;
	private ArrayBlockingQueue<DatagramPacket> sendingBuffer;
	private ConcurrentHashMap<Integer,Socket> sockets;
	private  ConcurrentHashMap<Integer, ConnectionInfo> ids;
	private SendingThread st;
	private ListenerThread lt;
	public Transport(int porta) throws SocketException{
		this.socketEntrada = new DatagramSocket(porta);
		this.socketSaida = new DatagramSocket();
		sendingBuffer = new ArrayBlockingQueue<DatagramPacket>(BUFFER_SIZE);
		sockets = new ConcurrentHashMap<Integer, Socket>();
		ids = new ConcurrentHashMap<Integer, ConnectionInfo>();
		this.st = new SendingThread(this);
		this.lt = new ListenerThread(this);
		lt.start();
		st.start();
	}
	public void kill(){
		st.kill();
		lt.kill();
	}
	public void addSocket(AppSocket sock, ConnectionInfo info){
		if(!sockets.containsKey(sock.getIdSocket())){
			sockets.put(new Integer(sock.getIdSocket()), sock);
		}
		ids.put(new Integer(sock.getIdSocket()), info);
		
	}
	public void addListenerSocket(Socket sock){
		sockets.put(new Integer(sock.getIdSocket()), sock);
	}
	public void removeSocket(AppSocket sock){
		sockets.remove(sock.id);
		ids.remove(sock.id);
	}
	public ConcurrentHashMap<Integer, Socket> getSockets(){
		return this.sockets;
	}

	public ArrayBlockingQueue<DatagramPacket> getBuffer(){
		return this.sendingBuffer;
	}
	public DatagramSocket getSocketSaida(){
		return this.socketSaida;
	}
	public DatagramSocket getSocketEntrada(){
		return this.socketEntrada;
	}
	/*
	 * 
	 * 1012 bytes + 12 bytes = 1024 bytes de tamanho maximo
	 */
	public void send(byte[] data, int id) {
		ConnectionInfo ci = ids.get(new Integer(id));
		if(ci != null){
			byte[] portaRetorno = ByteOp.intToByte(socketEntrada.getLocalPort());
			ByteBuffer bf = ByteBuffer.allocate(data.length+12);
			bf.put(ByteOp.intToByte(ci.getIdDestino()));
			bf.put(ByteOp.intToByte(id));
			bf.put(portaRetorno);
			bf.put(data);
			bf.flip();
			byte[] temp = bf.array();
			DatagramPacket pacote = new DatagramPacket(temp, temp.length, ci.getIp(), ci.getPortaDestino());
			sendingBuffer.offer(pacote);
		}
	}
	public void respond(ConnectionInfo destino){
		ByteBuffer bf = ByteBuffer.allocate(16);
		byte[] portaRetorno = ByteOp.intToByte(socketEntrada.getLocalPort());
		bf.put(ByteOp.intToByte(destino.getIdDestino()));
		bf.put(ByteOp.intToByte(-1));
		bf.put(portaRetorno);
		bf.flip();
		byte[] req = bf.array();
		DatagramPacket pacote = new DatagramPacket(req, req.length, destino.getIp(),destino.getPortaDestino());
		sendingBuffer.offer(pacote);
	}
	public void sendProbe(ConnectionInfo ci, int id){
		ByteBuffer bf = ByteBuffer.allocate(16);
		bf.put(ByteOp.intToByte(-1));
		bf.put(ByteOp.intToByte(id));
		bf.put(ByteOp.intToByte(socketEntrada.getLocalPort()));
		bf.flip();
		byte[] probe = bf.array();
		DatagramPacket pacote = new DatagramPacket(probe, probe.length, ci.getIp(), ci.getPortaDestino());
		sendingBuffer.offer(pacote);
	}
	public void sendRequest(ConnectionInfo destino,int id){
		ByteBuffer bf = ByteBuffer.allocate(16);
		byte[] portaRetorno = ByteOp.intToByte(socketEntrada.getLocalPort());
		bf.put(ByteOp.intToByte(destino.getIdDestino()));
		bf.put(ByteOp.intToByte(id));
		bf.put(portaRetorno);
		bf.flip();
		byte[] req = bf.array();
		DatagramPacket pacote = new DatagramPacket(req, req.length, destino.getIp(),destino.getPortaDestino());
		sendingBuffer.offer(pacote);
	}
	
}
